sbit LCD_RS at LATB2_bit;
sbit LCD_EN at LATB3_bit;
sbit LCD_D4 at LATB4_bit;
sbit LCD_D5 at LATB5_bit;
sbit LCD_D6 at LATB6_bit;
sbit LCD_D7 at LATB7_bit;

sbit LCD_RS_Direction at TRISB2_bit;
sbit LCD_EN_Direction at TRISB3_bit;
sbit LCD_D4_Direction at TRISB4_bit;
sbit LCD_D5_Direction at TRISB5_bit;
sbit LCD_D6_Direction at TRISB6_bit;
sbit LCD_D7_Direction at TRISB7_bit;

sbit Soft_I2C_Scl at RA3_bit;
sbit Soft_I2C_Sda at RA4_bit;
sbit Soft_I2C_Scl_Direction at TRISA3_bit;
sbit Soft_I2C_Sda_Direction at TRISA4_bit;

unsigned long   cntr ;          // number of RB0 transition
unsigned int    ovrflw ;        // number of timer0 overflows
unsigned char   str[10] ;       // display result string
char error;
void  Comm_Write(unsigned char *s);
void  interrupt(void);
void  Long2str(void);

void main( void)
{

  ADCON1 |= 0x0f;        // disable analog inputs

  error = Soft_UART_Init(&PORTA, 4, 3, 9600, 0); // Initialize Soft UART at 14400 bps   rx tx
     TRISB.B2 = 1 ;      //RB0 interrupt pin as input
      T0CON = 11001000;
     INTCON2 |= 10010000;            //interrupt on rising edge
      Delay_ms(2000);
     Comm_Write("Serial OK");

  do
  {
      cntr = 0 ;              // clear counters           
      ovrflw = 0 ;        // T0IF, INTF and GIE enabled
      INTCON = 10100000 ;           // T0IF, INTF and GIE enabled

      while(ovrflw < 39063) ;         // wait 1 second : 39062.5 = 40 000 000 / 4 / 256, rounded up
      INTCON.GIE = 0 ;                // stop all interrupts
      Long2Str() ;
      Comm_Write(str) ;               // write string      
  } while( 1);
}

void  interrupt(void)
   {    
     if(INTCON3.INT2IF)
       {                /*                 * RB0 interrupt                 */
          cntr++ ;                // inc. transition counter
          INTCON3.INT2IF = 0 ;       // clear interrupt flag to enable next call
       }
     else if(INTCON.TMR0IF)
       {                /** TIMER 0 overflow    */
          ovrflw++ ;              // inc. overflow counter
          INTCON.TMR0IF = 0 ;       // clear interrupt flag to enable next call on overflow
       }
   }
   
void  Long2str(void)
 {
    unsigned char   i, j ;     
    if(cntr == 0)
    {
      str[0] = '0' ;
      str[1] = 0 ;
    }
    else
    {
      str[0] = 0 ;
      i = 0 ;
      while(cntr > 0)                        
      {                        
        for(j = i + 1 ; j > 0 ; j--)                  
        {
          str[j] = str[j - 1] ;
        }
        str[0] = cntr % 10 ;
        str[0] += '0' ;
        i++ ;
        cntr /= 10 ;
      }
    }
 }
 
 void  Comm_Write(unsigned char *s)   
 {        
   while(*s)
   {
     Soft_Uart_Write(*s) ;
     s++ ;
   }
 }